"""Process that runs during Command-Line Interface usage"""

import datetime
import os
import logging
from pathlib import Path
import sys

from isoplot.main.dataprep import IsoplotData
from isoplot.ui.isoplotcli import IsoplotCli
from isoplot.ui.isoplot_notebook import check_version


def main():

    # We start by checking the version of isoplot before cli initialization
    check_version('isoplot')

    cli = IsoplotCli()
    cli.initialize_cli()
    # Initialize path to root directory (directory containing data file)
    cli.home = Path(cli.args.input_path).parents[0]

    # Get time and date for the run directory name
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%Hh%Mmn")

    # Initialize run name and run directory
    run_name = cli.args.run_name + "_" + date_time
    cli.run_home = cli.home / run_name
    cli.run_home.mkdir()

    # Prepare logger
    logger = logging.getLogger("Isoplot.isoplotcli")
    handle = logging.StreamHandler()
    fhandle = logging.FileHandler(cli.run_home / "debug.log") # Log debug output to file
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handle.setFormatter(formatter)
    fhandle.setFormatter(formatter)
    logger.addHandler(handle)
    logger.addHandler(fhandle)
    logger.setLevel(logging.DEBUG)
    handle.setLevel(logging.INFO)
    fhandle.setLevel(logging.DEBUG)

    logger.debug("Generate Data Object")

    try:
        data = IsoplotData(cli.args.input_path)
        data.get_data()

    except Exception as dataload_err:
        raise RuntimeError(f"Error while loading data. \n Error: {dataload_err}")

    if cli.args.generate_template:
        logger.debug("Generating template")
        try:
            data.generate_template()
        except Exception:
            logger.exception(f"There was an error while generating the template for the run {cli.args.run_name}.")
            sys.exit()
        else:
            logger.info(f"Template has been generated. Check destination folder at {cli.home}")
            sys.exit()

    os.chdir(cli.run_home)

    if hasattr(cli.args, 'template_path'):
        try:
            logger.debug("Loading template")
            data.get_template(cli.args.template_path)

            logger.debug("Merging data")
            data.merge_data()

            logger.debug("Preparing data")
            data.prepare_data()

        except Exception:
            logger.exception("There was a problem while loading the template")
            sys.exit()

    # Get lists of parameters for plots
    try:
        cli.metabolites = IsoplotCli.get_cli_input(cli.args.metabolite, "metabolite", data)
        cli.conditions = IsoplotCli.get_cli_input(cli.args.condition, "condition", data)
        cli.times = IsoplotCli.get_cli_input(cli.args.time, "time", data)

    except Exception:
        logger.exception("There was an error while parsing cli input information")

    logger.info("-------------------------------")
    logger.info("Cli has been initialized. Parameters are as follows")
    logger.info(f"Run name: {cli.args.run_name}")
    logger.info(f"Input data path: {cli.args.input_path}")
    logger.info(f"Template path: {cli.args.template_path}")
    logger.info(f"Chosen format: {cli.args.format}")
    logger.info(f"Data to plot: {cli.args.value}")
    logger.info(f"Chosen metabolites: {cli.metabolites}")
    logger.info(f"Chosen conditions: {cli.conditions}")
    logger.info(f"Chosen times: {cli.times}")
    logger.info("-------------------------------")

    logger.info("Creating plots...")
    try:
        cli.plot_figs(cli.metabolites, data, cli.args.zip)
    except Exception:
        logger.exception("There was a problem during the creation of the plots")
    else:
        logger.info("Plots created. Run is terminated")
        sys.exit()

if __name__ == "__main__":
    main()


import logging
import pathlib as pl

import pandas as pd
from natsort import natsorted


class IsoplotData:
    """
    Class to prepare Isoplot Data for plotting
    
    :param datapath: Path to .csv file containing Isocor output data
    :type datapath: str
    """

    def __init__(self, datapath):

        self.datapath = datapath
        self.data = None
        self.template = None
        self.dfmerge = None

        self.isoplot_logger = logging.getLogger("Isoplot.dataprep.IsoplotData")
        stream_handle = logging.StreamHandler()
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        stream_handle.setFormatter(formatter)
        self.isoplot_logger.addHandler(stream_handle)

        self.isoplot_logger.debug('Initializing IsoplotData object')


    @staticmethod
    def read_data(path, excel_sheet=0):
        """Function to read incoming data"""

        datapath = pl.Path(path)

        if datapath.suffix == ".tsv":
            data = pd.read_csv(datapath, sep="\t", engine='python')

        elif datapath.suffix == ".csv":
            data = pd.read_csv(datapath, sep=";", engine='python')

        elif datapath.suffix == ".xlsx":
            data = pd.read_excel(datapath, engine="openpyxl", sheet_name=excel_sheet)

        else:
            raise TypeError("File extension not supported."
                            "Supported types: '.tsv', '.csv' and '.xlsx' ")

        return data

    def get_data(self):
        """Read data from tsv file and store in object data attribute."""

        self.isoplot_logger.info(f'Reading datafile {self.datapath} \n')

        self.data = IsoplotData.read_data(self.datapath)

        self.isoplot_logger.info("Data is loaded")

    def generate_template(self):
        """Generate .xlsx template that user must fill"""

        self.isoplot_logger.info("Generating template...")

        metadata = pd.DataFrame(columns=[
            "sample", "condition", "condition_order", "time", "number_rep", "normalization"])
        metadata["sample"] = natsorted(self.data["sample"].unique())
        metadata["condition"] = 'votre_condition'
        metadata["condition_order"] = 1
        metadata["time"] = 1
        metadata["number_rep"] = 3
        metadata["normalization"] = 1.0
        metadata.to_excel(r'ModifyThis.xlsx', index=False)

        self.isoplot_logger.info('Template has been generated')

    def get_template(self, path):
        """Read user-filled template and catch any encoding errors"""

        self.isoplot_logger.info("Reading template...")

        try:
            self.isoplot_logger.debug('Trying to read excel template')
            self.template = pd.read_excel(path, engine='openpyxl')

        except UnicodeDecodeError as uni:
            self.isoplot_logger.error(uni)
            self.isoplot_logger.error(
                'Unable to read file. Check file encoding (must be utf-8) or file format (format must be .xlsx)')

        except Exception as err:
            self.isoplot_logger.error("There has been a problem...")
            self.isoplot_logger.error(err)

        else:
            self.isoplot_logger.info("Template succesfully loaded")

    def merge_data(self):
        """Merge template and data into pandas dataframe """

        self.isoplot_logger.info("Merging into dataframe...")

        try:
            self.isoplot_logger.debug('Trying to merge datas')
            self.dfmerge = self.data.merge(self.template)

            if not isinstance(self.dfmerge, pd.DataFrame):
                raise TypeError(
                    f"Error while merging data, dataframe not created. Data turned out to be {type(self.dfmerge)}")

        except Exception as err:
            self.isoplot_logger.error(err)
            self.isoplot_logger.error(
                'Merge impossible. Check column headers or file format (format must be .xlsx)')
            raise

        else:
            self.isoplot_logger.info('Dataframes have been merged')

    def prepare_data(self, export=True):
        """Final cleaning of data and export"""

        self.isoplot_logger.debug('Preparing data after merge: normalizing...')

        self.dfmerge["corrected area normalized"] = self.dfmerge["corrected_area"] / self.dfmerge["normalization"]
        self.dfmerge['metabolite'].drop_duplicates()

        self.isoplot_logger.debug("Creating IDs...")

        # Nous créons ici une colonne pour identifier chaque ligne avec condition+temps+numero de répétition
        # (possibilité de rajouter un tag metabolite plus tard si besoin)
        self.dfmerge.condition = self.dfmerge.condition.str.replace("_", "-")
        self.dfmerge['ID'] = self.dfmerge['condition'].apply(str) + '_T' + self.dfmerge['time'].apply(str) + '_' + \
                             self.dfmerge['number_rep'].apply(str)

        self.isoplot_logger.debug('Applying final transformations...')

        # Vaut mieux ensuite retransformer les colonnes temps et number_rep en entiers pour
        # éviter des problèmes éventuels de type
        self.dfmerge['time'].apply(int)
        self.dfmerge['number_rep'].apply(int)
        self.dfmerge.sort_values(['condition_order', 'condition'], inplace=True)
        self.dfmerge.fillna(0, inplace=True)
        if export:
            self.dfmerge.to_excel(r'Data Export.xlsx', index=False)
        self.isoplot_logger.info('Data exported. Check Data Export.xlsx')



""" Module for deploying pytest tests"""

import pytest
from pandas.api.types import is_numeric_dtype, is_string_dtype
from numpy import int64

from isoplot.main.dataprep import IsoplotData


@pytest.fixture(scope='function', autouse=True)
def df():
    df = IsoplotData(r".\test_data\160419_T_Daubon_MC_principale_res.csv")
    return df

@pytest.fixture(scope='function', autouse=True)
def columns():
    columns = ["sample", "metabolite", "area", "corrected_area", "isotopologue_fraction", "mean_enrichment"]
    return columns

@pytest.fixture(scope='function', autouse=True)
def sample_names():
    sample_names = ['110419_T0_Cont_1_2',
                    '110419_T0_Cont_2_8',
                    '110419_T0_Cont_3_9',
                    '110419_T24_Cont_1_27',
                    '110419_T24_Cont_2_28',
                    '110419_T24_Cont_3_29',
                    '110419_T48_Cont_1_47',
                    '110419_T48_Cont_2_48',
                    '110419_T48_Cont_3_49',
                    '110419_T0_A_1_11',
                    '110419_T0_A_2_12',
                    '110419_T0_A_3_13',
                    '110419_T24_A_1_31',
                    '110419_T24_A_2_32',
                    '110419_T24_A_3_33',
                    '110419_T48_A_1_51',
                    '110419_T48_A_2_52',
                    '110419_T48_A_3_53',
                    '110419_T0_B_1_15',
                    '110419_T0_B_2_16',
                    '110419_T0_B_3_17',
                    '110419_T24_B_1_35',
                    '110419_T24_B_2_36',
                    '110419_T24_B_3_37',
                    '110419_T48_B_1_55',
                    '110419_T48_B_2_56',
                    '110419_T48_B_3_57',
                    '110419_T0_AB_1_19',
                    '110419_T0_AB_2_20',
                    '110419_T0_AB_3_21',
                    '110419_T24_AB_1_39',
                    '110419_T24_AB_2_40',
                    '110419_T24_AB_3_41',
                    '110419_T48_AB_1_59',
                    '110419_T48_AB_2_60',
                    '110419_T48_AB_3_61']

    return sample_names


class Test_dataprep:

    def test_initial_df(self, df, columns, sample_names):

        df.get_data()

        assert not df.data.empty
        assert all(item in df.data.columns for item in columns)
        assert all(item in list(df.data["sample"]) for item in sample_names)

        for col in ["area", "corrected_area", "isotopologue_fraction", "mean_enrichment"]:
            assert is_numeric_dtype(df.data[col])
        for col in ["sample", "metabolite"]:
            assert is_string_dtype(df.data[col])

    def test_template(self, df, sample_names):

        df.get_template(r".\test_data\modified_for_testing.xlsx")

        assert not df.template.empty
        assert all(item in set(df.template["sample"]) for item in sample_names)
        assert is_numeric_dtype(df.template["time"])
        assert is_numeric_dtype(df.template["number_rep"])
        assert is_string_dtype(df.template["condition"])

    def test_merge(self, df):

        df.get_data()
        df.get_template(r".\test_data\modified_for_testing.xlsx")
        df.merge_data()

        assert hasattr(df, "dfmerge")

        for col in df.dfmerge.columns:
            if col in df.data.columns:
                assert df.data[col].sort_values().values.all() == df.dfmerge[col].sort_values().values.all()
            elif col in df.template.columns:
                assert df.template[col].sort_values().values.all() == df.dfmerge[col].sort_values().values.all()
            else:
                raise KeyError(f"{col} not found in columns")

    def test_prep_data(self, df):

        df.get_data()
        df.get_template(r".\test_data\modified_for_testing.xlsx")
        df.merge_data()
        df.prepare_data(False)

        assert ~df.dfmerge.isna().values.any()
        assert df.dfmerge["time"].dtype == int64
        assert df.dfmerge["number_rep"].dtype == int64
        for value in df.dfmerge["ID"]:
            assert type(value) is str

        for ids in df.dfmerge["ID"]:
            assert len(ids.split("_")) == 3


"""Module containing the CLI class that will be used during the cli process to get arguments from user and
generate the desired plots"""

import os
import argparse
import zipfile
import io

from bokeh.resources import CDN
from bokeh.embed import file_html

from isoplot.main.plots import StaticPlot, InteractivePlot, Map


class IsoplotCli:

    def __init__(self):

        self.parser = argparse.ArgumentParser("Isoplot2: Plotting isotopic labelling MS data")

        self.home = None
        self.run_home = None
        self.static_plot = None
        self.int_plot = None
        self.map = None
        self.args = None

        self.metabolites = []
        self.conditions = []
        self.times = []

    def dir_init(self, plot_type):
        """Initialize directory for plot"""
        wd = self.run_home / plot_type
        if os.path.exists(wd):
            os.chdir(wd)
        else:
            wd.mkdir()
            os.chdir(wd)

    def go_home(self):
        """Exit after work is done"""
        os.chdir(self.home)

    def parse_Args(self):
        """
        Parse arguments from user

        :return: Argument Parser object
        :rtype: class: argparse.ArgumentParser
        """

        self.parser.add_argument('input_path', help="Path to datafile")
        self.parser.add_argument("run_name", help="Name of the current run")
        self.parser.add_argument("format", help="Format of generated file")

        values = ['corrected_area', 'isotopologue_fraction', 'mean_enrichment']
        self.parser.add_argument('--value', choices=values, default='isotopologue_fraction',
                                 action="store", required=True, nargs='*',
                                 help="Select values to plot. This option can be given multiple times")

        self.parser.add_argument('-m', '--metabolite', default='all',
                                 help="Metabolite(s) to plot. For all, type in 'all' ")
        self.parser.add_argument('-c', '--condition', default='all',
                                 help="Condition(s) to plot. For all, type in 'all' ")
        self.parser.add_argument('-t', '--time', default='all',
                                 help="Time(s) to plot. For all, type in 'all' ")
        self.parser.add_argument("-gt", "--generate_template", action="store_true",
                                 help="Generate the template using datafile metadata")
        self.parser.add_argument("-tp", "--template_path", type=str,
                                 help="Path to template file")

        self.parser.add_argument('-sa', '--stacked_areaplot', action="store_true",
                                 help='Create static stacked areaplot')
        self.parser.add_argument("-bp", "--barplot", action="store_true",
                                 help='Create static barplot')
        self.parser.add_argument('-mb', '--meaned_barplot', action="store_true",
                                 help='Create static barplot with meaned replicates')

        self.parser.add_argument('-IB', '--interactive_barplot', action="store_true",
                                 help='Create interactive stacked barplot')
        self.parser.add_argument('-IM', '--interactive_meanplot', action="store_true",
                                 help='Create interactive stacked barplot with meaned replicates')
        self.parser.add_argument('-IS', '--interactive_areaplot', action="store_true",
                                 help='Create interactive stacked areaplot')

        self.parser.add_argument('-hm', '--static_heatmap', action="store_true",
                                 help='Create a static heatmap using mean enrichment data')
        self.parser.add_argument('-cm', '--static_clustermap', action="store_true",
                                 help='Create a static heatmap with clustering using mean enrichment data')
        self.parser.add_argument('-HM', '--interactive_heatmap', action="store_true",
                                 help='Create interactive heatmap using mean enrichment data')

        self.parser.add_argument('-s', '--stack', action="store_false",
                                 help='Add option if barplots should be unstacked')
        self.parser.add_argument('-v', '--verbose', action="store_true",
                                 help='Turns logger to debug mode')
        self.parser.add_argument('-a', '--annot', action='store_true',
                                 help='Add option if annotations should be added on maps')
        self.parser.add_argument('-z', '--zip', action="store_true",
                                 help="Add option to export plots in zip file")

    @staticmethod
    def get_cli_input(arg, param, data_object):
        """
        Function to get input from user and check for errors in spelling.
        If an error is detected input is asked once more.
        This function is used for galaxy implementation

        :param arg: list from which strings must be parsed
        :param param: name of what we are looking for
        :type param: str
        :param data_object: IsoplotData object containing final clean dataframe
        :type data_object: class: 'isoplot.dataprep.IsoplotData'

        :return: Desired string after parsing
        :rtype: list

        """

        if arg == "all":
            desire = data_object.dfmerge[param].unique()
        else:
            is_error = True

            while is_error:
                try:
                    # Cli gives list of strings, se we must make words of them
                    desire = [item for item in arg.split(",")]
                    # Checking input for typos
                    for item in desire:
                        if item == "all":
                            break
                        else:
                            if item not in data_object.dfmerge[param].unique():
                                raise KeyError(f"One or more of the chosen {param}(s) were not in list. "
                                               f"Please check and try again. Error: {item}")
                except Exception as e:
                    raise RuntimeError(f"There was a problem while reading input. Error: {e}")

                else:
                    is_error = False
        return desire

    @staticmethod
    def zip_export(figures, name):
        """
        Function to save figures in figure list to zip file (taken from
        https://stackoverflow.com/questions/55616877/save-multiple-objects-to-zip-directly-from-memory-in-python)

        :param figures: storage of figures and their respective file names in tuples: (name, fig)
        :type figures: list of tuples
        :param export_path: path to export location
        :type export_path: str
        :param name: name of the exported zip file
        :type name: str
        """

        zip_file_name = f"{name}.zip"
        print(f"Creating archive: {zip_file_name}")
        with zipfile.ZipFile(zip_file_name, mode="w") as zf:
            for fig_name, fig in figures:
                if fig_name.endswith("svg"):
                    buf = io.BytesIO()
                    fig.savefig(buf, format="svg")
                elif fig_name.endswith("html"):
                    html = file_html(fig, CDN, fig_name)
                    buf = io.StringIO(html)
                else:
                    buf = io.BytesIO()
                    fig.savefig(buf)
                print(f"Writing image {fig_name} in the archive")
                zf.writestr(fig_name, buf.getvalue())

    def plot_figs(self, metabolite_list, data_object, rtrn):
        """
        Function to control which plot methods are called depending on the
        arguments that were parsed

        :param metabolite_list: metabolites to be plotted
        :type metabolite_list: list of str
        :param data_object: object containing the prepared data
        :type data_object: class: 'isoplot.main.dataprep.IsoplotData'
        """

        if rtrn:
            figures = []

        for metabolite in metabolite_list:
            for value in self.args.value:
                self.static_plot = StaticPlot(self.args.stack, value, data_object.dfmerge,
                                              self.args.run_name, metabolite, self.conditions, self.times,
                                              self.args.format, display=False, rtrn=rtrn)

                self.int_plot = InteractivePlot(self.args.stack, value, data_object.dfmerge,
                                                self.args.run_name, metabolite, self.conditions, self.times,
                                                display=False, rtrn=rtrn)

                # STATIC PLOTS
                if self.args.stacked_areaplot:
                    plot_name = "Static_Areaplots"
                    if rtrn:
                        fig = self.static_plot.stacked_areaplot()
                        fname = plot_name + "_" + self.static_plot.static_fig_name
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.static_plot.stacked_areaplot()

                if self.args.barplot and not (value == "mean_enrichment"):
                    plot_name = "Static_barplots"
                    if rtrn:
                        fig = self.static_plot.barplot()
                        fname = plot_name + "_" + self.static_plot.static_fig_name
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.static_plot.barplot()

                if self.args.meaned_barplot and not (value == "mean_enrichment"):
                    plot_name = "Static_barplots_SD"
                    if rtrn:
                        fig = self.static_plot.mean_barplot()
                        fname = plot_name + "_" + self.static_plot.static_fig_name
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.static_plot.mean_barplot()

                if self.args.barplot and (value == "mean_enrichment"):
                    plot_name = "Static_barplots"
                    if rtrn:
                        fig = self.static_plot.mean_enrichment_plot()
                        fname = plot_name + "_" + self.static_plot.static_fig_name
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.static_plot.mean_enrichment_plot()

                if self.args.meaned_barplot and (value == "mean_enrichment"):
                    plot_name = "Static_barplots_SD"
                    if rtrn:
                        fig = self.static_plot.mean_enrichment_meanplot()
                        fname = plot_name + "_" + self.static_plot.static_fig_name
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.static_plot.mean_enrichment_meanplot()

                # INTERACTIVE PLOTS
                if self.args.interactive_barplot and not (value == "mean_enrichment"):
                    plot_name = "Interactive_barplots"
                    if rtrn:
                        fig = self.int_plot.stacked_barplot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.stacked_barplot()

                if self.args.interactive_barplot and not self.args.stack:
                    plot_name = "Interactive_unstacked_barplots"
                    if rtrn:
                        fig = self.int_plot.unstacked_barplot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.unstacked_barplot()

                if self.args.interactive_meanplot and not (value == "mean_enrichment"):
                    plot_name = "Interactive_barplots_SD"
                    if rtrn:
                        fig = self.int_plot.stacked_meanplot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.stacked_meanplot()

                if self.args.interactive_meanplot and not self.args.stack:
                    plot_name = "Interactive_barplots_SD"
                    if rtrn:
                        fig = self.int_plot.unstacked_meanplot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.unstacked_meanplot()

                if self.args.interactive_barplot and (value == "mean_enrichment"):
                    plot_name = "Interactive_barplots"
                    if rtrn:
                        fig = self.int_plot.mean_enrichment_plot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.mean_enrichment_plot()

                if self.args.interactive_meanplot and (value == "mean_enrichment"):
                    plot_name = "Interactive_barplots_SD"
                    if rtrn:
                        fig = self.int_plot.mean_enrichment_meanplot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.mean_enrichment_meanplot()

                if self.args.interactive_areaplot:
                    plot_name = "Interactive_stackplots"
                    if rtrn:
                        fig = self.int_plot.stacked_areaplot()
                        fname = plot_name + "_" + self.int_plot.filename
                        figures.append((fname, fig))
                    else:
                        self.dir_init(plot_name)
                        self.int_plot.stacked_areaplot()

        # MAPS
        self.map = Map(data_object.dfmerge, self.args.run_name, self.args.annot, self.args.format, rtrn=rtrn)

        if self.args.static_heatmap:
            plot_name = "static_heatmap"
            if rtrn:
                fig = self.map.build_heatmap()
                fname = plot_name + f".{self.map.fmt}"
                figures.append((fname, fig))
            else:
                self.dir_init(plot_name)
                self.map.build_heatmap()

        if self.args.static_clustermap:
            plot_name = "static_clustermap"
            if rtrn:
                fig = self.map.build_clustermap()
                fname = plot_name + f".{self.map.fmt}"
                figures.append((fname, fig))
            else:
                self.dir_init(plot_name)
                self.map.build_clustermap()

        if self.args.interactive_heatmap:
            self.map.fmt = "html"
            plot_name = "interactive_heatmap"
            if rtrn:
                fig = self.map.build_interactive_heatmap()
                fname = plot_name + f".{self.map.fmt}"
                figures.append((fname, fig))
            else:
                self.dir_init(plot_name)
                self.map.build_interactive_heatmap()

        if rtrn:
            IsoplotCli.zip_export(figures, self.args.run_name)

        self.go_home()

    def initialize_cli(self):
        """Launch argument parsing and perform checks"""

        self.parse_Args()
        self.args = self.parser.parse_args()

        # Check for typos and input errors
        valid_formats = ['png', 'svg', 'pdf', 'jpeg', 'html']
        forbidden_characters = ["*", ".", '"', "/", "\\", "[", "]", ":", ";", "|", ","]

        if not os.path.exists(self.args.input_path):
            raise RuntimeError(f"Input path does not lead to valid file. "
                               f"Please check path: {self.args.input_path}")

        if self.args.format not in valid_formats:
            raise RuntimeError("Format must be png, svg, pdf, jpeg or html")

        for char in forbidden_characters:
            if char in self.args.run_name:
                raise RuntimeError(f"Invalid character in run name. "
                                   f"Forbidden characters are: {forbidden_characters}")

        if self.args.template_path and not os.path.exists(self.args.template_path):
            raise RuntimeError(f"Template path does not lead to valid file. "
                               f"Please check path: {self.args.template_path}")



"""Module for controlling the Isoplot notebook"""

import io
import os
import logging
from pathlib import Path

import ipywidgets as widgets

mod_logger = logging.getLogger(f"isoplot.isoplot_notebook")

class IsoplotNb:

    def __init__(self, verbose=False):
        """ Initialize the widgets used in the dashboard and initial parameters """

        # Get home directory
        self.home = Path(os.getcwd())

        # Initiate child logger for class instances
        self.logger = logging.getLogger("isoplot.isoplot_notebook.IsoplotNb")
        handler = logging.StreamHandler()

        if verbose:
            handler.setLevel(logging.DEBUG)
        else:
            handler.setLevel(logging.INFO)

        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s', "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(formatter)

        if not self.logger.hasHandlers():
            self.logger.addHandler(handler)

        # Initiate all the widgets
        self.upload_data = widgets.FileUpload(
        accept='',  # Accepted file extension e.g. '.txt', '.pdf', 'image/*', 'image/*,.pdf'
        multiple=False,  # True to accept multiple files upload else False
        description="Upload Datafile"
        )



        self.upload_template = widgets.FileUpload(accept="",
                                             multiple=False,
                                             description="Upload Template")

        self.generate_template_btn = widgets.Button(description='Create Template')

        self.submit_template_btn = widgets.Button(description='Submit Template')

        self.metabolite_selector = widgets.SelectMultiple(options=self.metabolite_list,
                                                          value=self.metabolite_list[0],
                                                          description="Metabolites",
                                                          disabled=False)

        self.condition_selector = widgets.SelectMultiple(options=self.condition_list,
                                                          value=self.condition_list[0],
                                                          description="Conditions",
                                                          disabled=False)

        self.time_selector = widgets.SelectMultiple(options=self.time_list,
                                                          value=self.time_list[0],
                                                          description="Times",
                                                          disabled=False)

        self.mode_selector = widgets.Dropdown(options=["Static", "Interactive"],
                                              value="Static",
                                              description="Mode",
                                              disabled=False)

        self.plot_selector = widgets.Dropdown(options=self.plot_list,
                                              value=self.plot_list[0],
                                              description="Plot Type",
                                              disabled=False)

        self.run_name = widgets.Text(value="",
                                     placeholder="Input run name",
                                     description="Run Name",
                                     disabled=False)

        self.log_console = widgets.Output()

        self.preview_window = widgets.Output()


    def make_gui(self):

        display(self.preview)




import io
import datetime
import os
import subprocess
import sys

import ipywidgets as widgets
import pandas as pd

from isoplot.main.dataprep import IsoplotData
from isoplot.main.plots import StaticPlot, InteractivePlot, Map


class ValueHolder:
    x: int = None


vh = ValueHolder()

# Check if current version is outdated
def check_version(name):
    reqs = subprocess.check_output([sys.executable, '-m', 'pip', 'list','--outdated'])
    outdated_packages = [r.decode().split('==')[0] for r in reqs.split()]
    if name in outdated_packages:
        print("Your version of Isoplot is outdated. Please run 'pip install isoplot' to get the latest"
              "version.")

# Instanciation des widgets
def make_uploader():
    global uploader
    uploader = widgets.FileUpload(
        accept='',  # Accepted file extension e.g. '.txt', '.pdf', 'image/*', 'image/*,.pdf'
        multiple=False  # True to accept multiple files upload else False
    )

    return uploader


def make_mduploader():
    global mduploader
    mduploader = widgets.FileUpload(
        accept='',  # Accepted file extension e.g. '.txt', '.pdf', 'image/*', 'image/*,.pdf'
        multiple=False  # True to accept multiple files upload else False
    )

    return mduploader

check_version('isoplot')

metadatabtn = widgets.Button(description='Create Template')

datamerge_btn = widgets.Button(description='Submit Template')

out = widgets.Output()

out2 = widgets.Output()


def metadatabtn_eventhandler(event):
    global data_object

    with out:
        print("Loading file...")

    uploaded_filename = next(iter(uploader.value))
    content = uploader.value[uploaded_filename]['content']
    with open('myfile', 'wb') as f: f.write(content)

    data_object = IsoplotData(None)
    key = list(uploader.value.keys())

    if key[0][-3:] == "tsv":
        data_object.data = pd.read_csv(io.BytesIO(content), sep="\t")
    elif key[0][-3:] == "csv":
        data_object.data = pd.read_csv(io.BytesIO(content), sep=";")

    data_object.generate_template()

    with out:
        print("Done!")


def dataprep_eventhandler(event):
    with out2:
        print('Loading file...')

    mduploaded_filename = next(iter(mduploader.value))
    content = mduploader.value[mduploaded_filename]['content']
    with open('myfile', 'wb') as f: f.write(content)

    data_object.get_template(io.BytesIO(content))
    data_object.merge_data()
    data_object.prepare_data()
    vh.dfmerge = data_object.dfmerge

    with out2:
        print("Done!")


# Gestion des événements pour les boutons
metadatabtn.on_click(metadatabtn_eventhandler)
datamerge_btn.on_click(dataprep_eventhandler)


# Fonction permettant le filtrage des données à plotter et appelant les fonctions de plotting
def indiplot(stack, value, data, name, metabolites, conditions, times, fmt, display, stackplot=False):
    # Préparons le directory où seront enregistrés les html avec les plots
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S")  # Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time)  # Créons le dir
    os.chdir(name + " " + date_time)  # Rentrons dans le dir

    for metabolite in metabolites:

        plotter = StaticPlot(stack, value, data, name, metabolite, conditions, times, fmt,
                             display=display)

        if value != 'mean_enrichment':
            if stackplot == True:
                plotter.stacked_areaplot()
            else:
                plotter.barplot()
        elif value == 'mean_enrichment':
            plotter.mean_enrichment_plot()

    os.chdir(mydir)  # Revenir au dir initial


# Fonction permettant le filtrage des données à plotter et appelant les fonctions de plotting
def meanplot(stack, value, data, name, metabolites, conditions, times, fmt, display):
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S")  # Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time)  # Créons le dir
    os.chdir(name + " " + date_time)  # Rentrons dans le dir

    for metabolite in metabolites:

        plotter = StaticPlot(stack, value, data, name, metabolite, conditions, times, fmt,
                             display=display)

        if value != 'mean_enrichment':
            plotter.mean_barplot()
        elif value == 'mean_enrichment':
            plotter.mean_enrichment_meanplot()

    os.chdir(mydir)  # Revenir au dir initial


# Création d'une fonction pour gérer les appels aux fonctions de plotting en individuel
def indibokplot(stack, value, data, name, metabolites, conditions, times, display, stackplot=False):
    # Préparons le directory où seront enregistrés les html avec les plots
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S")  # Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time)  # Créons le dir
    os.chdir(name + " " + date_time)  # Rentrons dans le dir

    for metabolite in metabolites:

        if name == '':
            name = metabolite

        plotter = InteractivePlot(stack, value, data, name, metabolite, conditions, times,
                                  display=display)

        # Le cas du mean enrichment est différent car les valeurs sont en double à la sortie d'Isocor
        if value != 'mean_enrichment':

            if stackplot == True:
                plotter.stacked_areaplot()

            elif stack == False:
                plotter.unstacked_barplot()

            elif stack == True:
                plotter.stacked_barplot()

        elif value == 'mean_enrichment':
            plotter.mean_enrichment_plot()

    os.chdir(mydir)  # Revenir au dir initial


# Création d'une fonction pour gérer les appels aux fonctions de plotting en individuel
def meanbokplot(stack, value, data, name, metabolites, conditions, times, display):
    # Préparons le directory où seront enregistrés les html avec les plots
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S")  # Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time)  # Créons le dir
    os.chdir(name + " " + date_time)  # Rentrons dans le dir

    for metabolite in metabolites:

        if name == '':
            name = metabolite

        plotter = InteractivePlot(stack, value, data, name, metabolite, conditions, times,
                                  display=display)

        if value != 'mean_enrichment':  # Le cas du mean enrichment est différent car les valeurs sont en double à la sortie d'Isocor

            if stack == False:
                plotter.unstacked_meanplot()

            elif stack == True:
                plotter.stacked_meanplot()

        elif value == 'mean_enrichment':
            plotter.mean_enrichment_meanplot()

    os.chdir(mydir)  # Revenir au dir initial


# Fontion pour choisir le map à générer:
def build_map(data, name, map_select, annot, fmt, display):
    mapper = Map(data, name, annot, fmt, display=display)

    if map_select == "Static heatmap":
        mapper.build_heatmap()

    if map_select == 'Interactive heatmap':
        mapper.fmt = 'html'
        mapper.build_interactive_heatmap()

    if map_select == "Clustermap":
        mapper.build_clustermap()
